#ifndef SPVM_EXTENTION_BIND_H
#define SPVM_EXTENTION_BIND_H

#include "spvm_base.h"

void SPVM_EXTENTION_BIND_bind_core_extentions(SPVM_COMPILER* compiler, SPVM_SUB* sub, const char* sub_abs_name);

#endif
